/*
 * Copyright (c) 2002 Patrick Tullmann
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this file for any
 * purpose with or without restriction is hereby granted.
 */

import java.io.*;

public abstract class Command
{
	private static final class Move extends Command
	{
		private Direction d = null;

		Move()
		{
			super("Move");
		}
		
		void setDir(Direction d)
		{
			this.d = d;

		}

		public void writeTo(PrintWriter pw)
			throws IOException
		{
			// XXX
			pw.println(this.bid + " Move " + this.d.dir);
		}
		
		public String toString()
		{
			return "Move " + this.d.dir;
		}
	}
	
	private static final class PkgCmd extends Command
	{
		private int[] ids = null;
		private int len;

		PkgCmd(String cmd)
		{
			super(cmd);
		}
		
		void setIds(int[] ids, int len)
		{
			if ((ids != null) 
			    && (len > ids.length))
				throw new Error("Bogus setIds");
			this.ids = ids;
			this.len = len;
		}

		public void writeTo(PrintWriter pw)
			throws IOException
		{
			pw.print(this.bid + " " + this.cmd);
			if (this.ids != null)
			{
				for (int i = 0; i < this.len; i++)
				{
					pw.print(" ");
					pw.print(this.ids[i]);
				}
			}
			pw.println();
		}

		public String toString()
		{
			if (this.ids == null)
				return this.cmd + " <none>";

			StringBuffer sbuf = new StringBuffer(10 + this.ids.length * 5);
			sbuf.append(this.cmd)
				.append(" <");
			String spacer = "";
			for (int i = 0; i < this.len; i++)
			{
				sbuf.append(spacer)
					.append(Integer.toString(this.ids[i]));
				spacer = " ";
			}
			sbuf.append(">");

			return sbuf.toString();
		}
	}
	
	private static Move m = new Move();
	private static PkgCmd p = new PkgCmd("Pick");
	private static PkgCmd d = new PkgCmd("Drop");

	protected int bid;
	protected String cmd;

	private Command(String cmd)
	{
		this.cmd = cmd;
	}

	void setBid(int bid)
	{
		this.bid = bid;
	}

	public static Command move(int bid, Direction d)
	{
		m.setBid(bid);
		m.setDir(d);
		return m;
	}

	public static Command pick(int bid, int[] ids, int max)
	{
		p.setBid(bid);
		p.setIds(ids, max);
		return p;
	}

	public static Command pick(int bid, int[] ids)
	{
		return pick(bid, ids, ids.length);
	}

	public static Command pick(int bid, Package pkg)
	{
		p.setBid(bid);
		p.setIds(new int[] { pkg.id() }, 1);
		return p;
	}

	public static Command noop(int bid)
	{
		p.setBid(bid);
		p.setIds(null, 0);
		return p;
	}

	public static Command drop(int bid, int[] ids)
	{
		d.setBid(bid);
		d.setIds(ids, ids.length);
		return d;
	}

	public static Command drop(int bid, Package pkg)
	{
		d.setBid(bid);
		d.setIds(new int[] { pkg.id() }, 1);
		return d;
	}

	public abstract void writeTo(PrintWriter pw)
		throws IOException;
}
