/*
 * Copyright (c) 2002 Patrick Tullmann
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this file for any
 * purpose with or without restriction is hereby granted.
 */

/**
 * Packages:
 *   - known
 *   - being carried by other (unknown stats)
 *   - dropped by other (unknown stats)
 */
public final class Package
implements IntKeyedHashtable.IntKey
{
	private final int id;
	private int weight; // and score value
	
	// Location of package (-1 if unknown)
	private int x;
	private int y;

	// If not being carried:
	private int destX;
	private int destY;
	
	private boolean delivered;

	// For tables:
	public int keyInt()
	{
		return this.id;
	}

	// For packages we don't know anything about
	public Package(int id)
	{
		this.id = id;
		this.weight = -1;
		this.destX = -1;
		this.destY = -1;
		this.delivered = false;
	}

	// For packages we've stood over
	public Package(int id, int destX, int destY, int w)
	{
		this.id = id;
		this.weight = w;

		this.destX = destX;
		this.destY = destY;
	}

	int weight()
	{
		return this.weight;
	}

	int destX()
	{
		return this.destX;
	}
	
	int destY()
	{
		return this.destY;
	}

	int id()
	{
		return this.id;
	}
	
	int x()
	{
		return this.x;
	}
	
	int y()
	{
		return this.y;
	}

	boolean delivered()
	{
		return this.delivered;
	}

	void setLocation(int x, int y)
	{
		this.x = x;
		this.y = y;

		if ((this.x == this.destX)
		    && (this.y == this.destY))
		{
			this.delivered = true;
		}
	}

	void update(Package p)
	{
		if (p.id() != this.id)
			throw new Error("updating with wrong package!");
		this.weight = p.weight;
		this.destX = p.destX;
		this.destY = p.destY;
	}

	public int hashCode()
	{
		return this.id;
	}

	public boolean equals(Object other)
	{
		if (other instanceof Package)
		{
			return (((Package)other).id == this.id);
		}
		return false;
	}

	public String toString()
	{
		return "Pkg[id=" +this.id+ "; w=" +this.weight+ "; d=(" +this.destX+ "," +this.destY+ ")]";
	}
}
