/*
 * Copyright (c) 2002 Patrick Tullmann
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this file for any
 * purpose with or without restriction is hereby granted.
 */

public abstract class Robot
{
	protected int id;
	protected int capacity; // maximum
	protected int money;

	protected int x;
	protected int y;

	protected int remainingCapacity;

	static Robot make(String type)
	{
		try
		{
			Class roboClass = Class.forName("Robot_" +type);
			Robot r = (Robot)roboClass.newInstance();
			return r;
		}
		catch (Exception e)
		{
			throw new Error("Blew up looking for `Robot_" +type+ "'", e);
		}
	}

	public Robot()
	{
		this.id = -1;
		this.capacity = -1;
		this.remainingCapacity = -1;
		this.money = -1;
		this.x = -1;
		this.y = -1;
	}

	int useFunds(int i)
	{
		int cost = i;

		if (i < 0)
			cost = -i;

//		if (this.money < cost)
//			throw new Error("Spending more than I've got...");

		this.money -= cost;
		return i;
	}

	final int x()
	{
		return this.x;
	}
	
	final int y()
	{
		return this.y;
	}

	final int funds()
	{
		return this.money;
	}

	public int id()
	{
		return this.id;
	}

	public final void setId(int id)
	{
		if (this.id != -1)
			throw new Error("reset id");
		this.id = id;
	}
	
	public final void setCap(int cap)
	{
		if (this.capacity != -1)
			throw new Error("reset cap");
		this.capacity = cap;
		this.remainingCapacity = cap;
	}

	public boolean canCarry(int weight)
	{
		return (this.remainingCapacity > weight);
	}

	public final void setMoney(int money)
	{
		if (this.money != -1)
			throw new Error("reset $");
		msg("FUNDS: " +money);
		this.money = money;
	}

	public void placed(int x, int y)
	{
		this.x = x;
		this.y = y;
	}
	
	public void picked(Package pkg)
	{
		msg("Yeah! Got package " +pkg+ "!");
		this.remainingCapacity -= pkg.weight();
		// XXX don't assume drop/pick notices in a particular order?
	}
	
	public void dropped(Package pkg)
	{
		msg("Dropped pkg " +pkg);
		this.remainingCapacity += pkg.weight();
		// XXX don't assume drop/pick notices in a particular order?
	}
	
	protected final void msg(String msg)
	{
		//System.err.println(msg);
	}

	public void moved(Direction d)
	{
		this.x += d.dx();
		this.y += d.dy();
	}

	public abstract Command command(Board board,
					WorldState worldState,
					Package[] currentPackages);
}
